print("import: 'distconfig'")
import distconfig

print("import: 'distconfig.backends'")
import distconfig.backends

print("import: 'distconfig.tests'")
import distconfig.tests

print("import: 'distconfig.tests.integration'")
import distconfig.tests.integration

print("import: 'distconfig.tests.unit'")
import distconfig.tests.unit

